%--------------------------------------------------------------------------
% Figure 2 - Regularization approaches
%--------------------------------------------------------------------------
% This DeerLab script was used to make Fig.2 of the paper: 
%
% Luis Fabregas Ibanez, Gunnar Jeschke, Stefan Stoll 
% DeerLab: A comprehensive toolbox for analyzing dipolar EPR spectroscopy data
% Magnetic Resonance, 2020
%
% DeerLab version: 
%   Release: 0.9.0
%--------------------------------------------------------------------------

clear,clc,clf

% Figure 2a
%---------------------------------------------------------------------

%Fix random numbers for reproduceable results
rng(0)

%Generate ground truth distance distribution
[rfine,P] = groundtruth('figure2');

%Define a shorter distance axis for the fit
r = linspace(2.5,5,300);


%Simulate dipolar signal
t = linspace(-0.5,4,500);
lambda = 0.2;
V = dipolarsignal(t,rfine,P,lambda,'noiselevel',0.01);

%Prepare dipolar kernel
K = dipolarkernel(t,r,lambda);

%Determine regularization parameter for to avoid computing twice
alpha = selregparam(V,K,'tikh','aic');

%Get Tikhonov fit
Ptikh = fitregmodel(V,K,r,'tikh',alpha);
Vtikh = K*Ptikh;

%Get TV fit
Ptv = fitregmodel(V,K,r,'tv','aic');
Vtv = K*Ptv;

%Get OBIR fit
Pobir = obir(V,K,r,'tikh',alpha,'NoiseLevelAim',0.01);
Vfit = K*Pobir;

%Plot results

subplot(221)
plot(t,V,'k.',t,Vfit,t,Vtikh,t,Vtv,'LineWidth',1.5)
axis tight
set(gca,'fontsize',13)
xlabel('Time [\mus]'),ylabel('V(t) [\mus^{-1}]')

subplot(222)
plot(rfine,P,'k',r,Pobir,r,Ptikh,'-',r,Ptv,'-','LineWidth',1.5)
axis tight
xlim([3 4.5])
set(gca,'fontsize',13)
xlabel('Distance [nm]'),ylabel('P(r) [nm^{-1}]')
legend('truth','OBIR','Tikh','TV')

% Figure 2b
%---------------------------------------------------------------------

rng(9)

%Simulate dipolar signal
t = linspace(-0.5,4,600);
lambda = 0.2;
V = dipolarsignal(t,rfine,P,lambda,'noiselevel',0.025);

%Prepare dipolar kernel
K = dipolarkernel(t,r,lambda);

%Determine regularization parameter for to avoid computing twice
alpha = selregparam(V,K,'tikh','aic');

%Get Tikhonov fit
Ptikh = fitregmodel(V,K,r,'tikh',alpha);
Vtikh = K*Ptikh;

%Get TV fit
Ptv = fitregmodel(V,K,r,'tv','aic');
Vtv = K*Ptv;

%Get OBIR fit
Pobir = obir(V,K,r,'tikh',alpha,'NoiseLevelAim',0.025);
Vfit = K*Pobir;

%Plot results

subplot(223)
plot(t,V,'k.',t,Vfit,t,Vtikh,t,Vtv,'LineWidth',1.5)
axis tight
set(gca,'fontsize',13)
xlabel('Time [\mus]'),ylabel('V(t) [\mus^{-1}]')

subplot(224)
plot(rfine,P,'k',r,Pobir,r,Ptikh,'-',r,Ptv,'-','LineWidth',1.5)
axis tight
xlim([3 4.5])
set(gca,'fontsize',13)
xlabel('Distance [nm]'),ylabel('P(r) [nm^{-1}]')
legend('truth','OBIR','Tikh','TV')