# %%
"""
------------------------------------------------------------------------------
Figure 1 - Regularization approaches
------------------------------------------------------------------------------
This DeerLab script was used to make Fig.1 of the paper: 

Luis Fabregas Ibanez, Gunnar Jeschke, Stefan Stoll 
DeerLab: A comprehensive toolbox for analyzing dipolar EPR spectroscopy data
Magnetic Resonance, 2020

DeerLab version: 
Release: 0.10.0
------------------------------------------------------------------------------
"""
import codecs
import numpy as np
import matplotlib.pyplot as plt
import deerlab as dl



#%%
# Figure 1a
#---------------------------------------------------------------------

# Ground truth
data = np.genfromtxt("./data/figure1a_Pdata.txt", dtype=None, skip_header=1)
rtruth,Ptruth = [data[:,i] for i in range(2)]

# Input signal
data = np.genfromtxt("./data/figure1a_Vdata.txt", dtype=None, skip_header=1)
t,V = [data[:,i] for i in range(2)]

#Define a shorter distance axis for the fit
r = np.linspace(2.5,5,300)

#Prepare dipolar kernel
lamb = 0.2 # we assume that we know the true modulation depth
K = dl.dipolarkernel(t,r,lamb)

#Determine regularization parameter to avoid computing twice
alpha = dl.selregparam(V,K,r,'tikhonov','aic')

#Get Tikhonov fit
TIKHfit = dl.fitregmodel(V,K,r,'tikhonov',alpha)
Ptikh = TIKHfit.P
Vtikh = K@Ptikh

#Get TV fit
TVfit = dl.fitregmodel(V,K,r,'tv','aic')
Ptv = TVfit.P
Vtv = K@Ptv

#Get OBIR fit
OBIRfit = dl.fitregmodel(V,K,r,'tikhonov',alpha, obir=True, noiselevelaim=0.01)
Pobir = OBIRfit.P
Vobir = K@Pobir

#Plot results
plt.subplot(121)
plt.plot(t,V,'.',color='gray',alpha=0.4)
plt.plot(t,Vobir,t,Vtikh,t,Vtv, linewidth=1.5)
plt.xlabel('Time [$\mu s$]')
plt.ylabel('V(t)')

plt.subplot(122)
plt.fill(rtruth,Ptruth,color='gray',alpha=0.4)
plt.plot(r,Pobir,r,Ptikh,'-',r,Ptv,'-', linewidth=1.5)
plt.xlim([3, 4.5])
plt.xlabel('Distance [nm]')
plt.ylabel('P(r) [nm$^{-1}$]')
plt.legend(['OBIR','Tikh','TV','truth'])


#%%

# Figure 1b
#---------------------------------------------------------------------

# Ground truth
data = np.genfromtxt("./data/figure1b_Pdata.txt", dtype=None, skip_header=1)
rtruth,Ptruth = [data[:,i] for i in range(2)]

# Input signal
data = np.genfromtxt("./data/figure1b_Vdata.txt", dtype=None, skip_header=1)
t,V = [data[:,i] for i in range(2)]

#Prepare dipolar kernel
lamb = 0.2 # we assume that we know the true modulation depth
K = dl.dipolarkernel(t,r,lamb)

#Determine regularization parameter to avoid computing twice
alpha = dl.selregparam(V,K,r,'tikhonov','aic')

#Get Tikhonov fit
TIKHfit = dl.fitregmodel(V,K,r,'tikhonov',alpha)
Ptikh = TIKHfit.P
Vtikh = K@Ptikh

#Get TV fit
TVfit = dl.fitregmodel(V,K,r,'tv','aic')
Ptv = TVfit.P
Vtv = K@Ptv

#Get OBIR fit
OBIRfit = dl.fitregmodel(V,K,r,'tikhonov',alpha, obir=True, noiselevelaim=0.025)
Pobir = OBIRfit.P
Vobir = K@Pobir
#%%
#Plot results
plt.subplot(121)
plt.plot(t,V,'.',color='gray',alpha=0.4)
plt.plot(t,Vobir,t,Vtikh,t,Vtv, linewidth=1.5)
plt.xlabel('Time [$\mu s$]')
plt.ylabel('V(t)')

plt.subplot(122)
plt.fill(rtruth,Ptruth,color='gray',alpha=0.4)
plt.plot(r,Pobir,r,Ptikh,'-',r,Ptv,'-', linewidth=1.5)
plt.xlim([3, 4.5])
plt.xlabel('Distance [nm]')
plt.ylabel('P(r) [nm$^{-1}$]')
plt.legend(['OBIR','Tikh','TV','truth'])


# %%
