# %%
"""
--------------------------------------------------------------------------
Figure 2 - Multi-Gauss model fitting
--------------------------------------------------------------------------
This DeerLab script was used to make Fig.2 of the paper: 

Luis Fabregas Ibanez, Gunnar Jeschke, Stefan Stoll 
DeerLab: A comprehensive toolbox for analyzing dipolar EPR spectroscopy data
Magnetic Resonance, 2020

DeerLab version: 
Release: 0.10.0
--------------------------------------------------------------------------
"""

import numpy as np
import matplotlib.pyplot as plt
import deerlab as dl
from deerlab.dd_models import dd_gauss
from deerlab.bg_models import bg_hom3d

# Load data
#-------------------------
# Ground truth
data = np.genfromtxt("./data/figure2_Pdata.txt", dtype=None, skip_header=1)
r,Ptruth = [data[:,i] for i in range(2)]

# Input signal
data = np.genfromtxt("./data/figure2_Vdata.txt", dtype=None, skip_header=1)
t,V = [data[:,i] for i in range(2)]


# Run multi-Gauss fitting
#-------------------------

# Definition of a parametric dipolar kernel for the description of a 4-pulse DEER experiment.
def Kmodel(par):
    # Unpack parameters
    lam,conc = par
    # Simualte background
    B = bg_hom3d(t,conc,lam)
    # Generate dipolar kernel
    K = dl.dipolarkernel(t,r,lam,B)
    return K

# Parameter bounds:
#     rmean fwhm
lb  = [1,  0.2] # distribution basis function lower bounds
ub  = [20,  3]    # distribution basis function upper bounds
#     lambda conc  
lbK = [0.1, 0.05]  # kernel parameters lower bounds
ubK = [0.5, 5000]  # kernel parameters upper bounds

# Prepare the kernel model
maxGauss = 5 # maximum number of Gaussians

# Fit the kernel parameters with an optimized multi-Gauss distribution
fit = dl.fitmultimodel(V,Kmodel,r,dd_gauss,maxGauss,'aic',lb,ub,lbK,ubK,multistart=30,uqanalysis=False)
#Extract results
Pfit = fit.P
Kparfit = fit.Kparam
metrics = fit.selfun
Peval = fit.Pn

# Get the time-domain fit
K = Kmodel(Kparfit)
Vfit = K@Pfit

# Compute the Akaike weights
dAIC = metrics - min(metrics)
Akaikeweights = 100*np.exp(-dAIC/2)/sum(np.exp(-dAIC/2))

# Plot results
#-------------------------
plt.figure(figsize=(10,8))

plt.subplot(3,2,1)
plt.plot(t,V,'k.')
plt.plot(t,Vfit,'b',linewidth=1.5)
plt.plot(t,(1-Kparfit[0])*bg_hom3d(t,Kparfit[1],Kparfit[0]),'b--',linewidth=1.5)
plt.tight_layout()
plt.grid(alpha=0.3)
plt.legend(['data','Vfit','Bfit'])
plt.xlabel('t [$\mu s$]')
plt.ylabel('V(t)')

plt.subplot(322)
plt.fill(r,Ptruth,color='gray',alpha=0.4)
plt.plot(r,Pfit,'b',linewidth=1.5)
plt.tight_layout()
plt.grid(alpha=0.3)
plt.legend(['truth','optimal fit','95%-CI'])
plt.xlabel('r [nm]')
plt.ylabel('P(r)')

plt.subplot(323)
plt.bar(np.arange(maxGauss)+1,metrics + abs(min(metrics)),facecolor='b',alpha=0.6)
plt.tight_layout()
plt.grid(alpha=0.3)
plt.ylabel('$\Delta AIC$')
plt.xlabel('Number of Gaussians')

plt.subplot(325)
plt.bar(np.arange(maxGauss)+1,Akaikeweights,facecolor='b',alpha=0.6)
plt.tight_layout()
plt.grid(alpha=0.3)
plt.ylabel('Akaike Weight [%]')
plt.xlabel('Number of Gaussians')

plt.subplot(3,2,(4,6))
for i in range(len(Peval)):
    plt.plot(r,Ptruth + i+1,'k',r,Peval[i] + i+1,'b-',linewidth=1.5)
plt.tight_layout()
plt.grid(alpha=0.3)
plt.xlabel('r [nm]')
plt.ylabel('Number of Gaussians')
plt.legend(['truth','fit'])

# %%
