%--------------------------------------------------------------------------
% Figure 5 - One-step vs Two-step Analysis
%--------------------------------------------------------------------------
% This DeerLab script was used to make Fig.5 of the paper: 
%
% Luis Fabregas Ibanez, Gunnar Jeschke, Stefan Stoll 
% DeerLab: A comprehensive toolbox for analyzing dipolar EPR spectroscopy data
% Magnetic Resonance, 2020
%
% DeerLab version: 
%   Release: 0.9.0
%--------------------------------------------------------------------------

%Fix random numbers for reproduceable results
rng(1)

% Load example data
[r,P] = groundtruth('figure5');

% Loop over all four cases of truncation
for i=1:4

    switch i
        case 1
            % Plot 1: truncation 1/4
            t = linspace(-0.5,8,300);
        case 2
            % Plot 2: truncation 2/4
            t = linspace(-0.5,3.2,300);
        case 3
            % Plot 3: truncation 3/4
            t = linspace(-0.5,1.5,300);
        case 4
            % Plot 4: truncation 4/4
            t = linspace(-0.5,0.7,300);
    end
    
    % Simulate the dipolar signal

    B = bg_exp(t,0.09);
    lam = 0.3;
    rng(1)
    V = dipolarsignal(t,r,P,lam,B) + whitegaussnoise(t,0.005,'rescale');
    
    % Create function handle depending on r and param from the custom model
    fcnhandle = @(t,param) mymodel(t,param,r,V);
    
    % One-step analysis
    %--------------------------------------------------------------------------
    % Fit the background and distribution simultaneously
    parafit = fitparamodel(V,fcnhandle,t,[0.05,0.05],'Lower',[0 0.01],'Upper',[1 1],'TolFun',1e-5);
    [Vfit,Pfit,Bfit,alphaopt] = mymodel(t,parafit,r,V);
    Bfit = (1-parafit(1))*Bfit;    
    
    % Two-step analysis
    %--------------------------------------------------------------------------
    % Fit the background...
    [Bfit2,lam2,~,tstart] = fitbackground(V,t,@bg_exp);
    tstart
    KB2 = dipolarkernel(t,r,lam2,Bfit2);
    % ...and then the distribution
    Pfit2 = fitregmodel(V,KB2,r,'tikhonov','aic');
    Vfit2 = KB2*Pfit2;
    
    % Plot the results
    %--------------------------------------------------------------------------
    subplot(3,4,i)
    plot(t,V,'k.',t,Vfit,'b-',t,Bfit,'b--',t,(1-lam)*B,'k--','LineWidth',1.5)
    axis tight
    set(gca,'fontsize',14)
    ylims = ylim;
    xlabel('t [\mus]')
    ylabel('V(t)')
    
    subplot(3,4,i+4)
    plot(t,V,'k.',t,Vfit2,'r-',t,(1-lam2)*Bfit2,'r--',t,(1-lam)*B,'k--','LineWidth',1.5)
    axis tight
    set(gca,'fontsize',14)
    ylim(ylims)
    xlabel('t [\mus]')
    ylabel('V(t)')
        
    subplot(3,4,i+8)
    plot(r,P,'k',r,Pfit,'b',r,Pfit2,'r','LineWidth',1.5)
    axis tight
    xlim([3.5 6.5])
    set(gca,'fontsize',14)
    xlabel('r [nm]')
    ylabel('P(r) [nm^{-1}]')
    
    drawnow
end

% Definition of the parametric dipolar signal model
%----------------------------------------------
function [Vfit,Pfit,Bfit,alphaopt] = mymodel(t,param,r,V)

% Extract parameters from the upper-level
lambda = param(1);
k = param(2);

% Generate the fitted kernel
Bfit = bg_exp(t,k);
K = dipolarsignal(t,r,lambda,Bfit);

% Find the solution of the lower-level via Tikhonov regularization
[Pfit,alphaopt] = fitregmodel(V,K,r,'tikhonov','aic');

% Get the time-domain fit
Vfit = K*Pfit;

end
