# %%
""" 
--------------------------------------------------------------------------
Figure 4 - One-step vs Two-step Analysis
--------------------------------------------------------------------------
This DeerLab script was used to make Fig.4 of the paper: 

Luis Fabregas Ibanez, Gunnar Jeschke, Stefan Stoll 
DeerLab: A comprehensive toolbox for analyzing dipolar EPR spectroscopy data
Magnetic Resonance, 2020

DeerLab version: 
Release: 0.10.0
--------------------------------------------------------------------------
"""

import numpy as np
import matplotlib.pyplot as plt
import deerlab as dl
from deerlab.ex_models import ex_4pdeer
from deerlab.bg_models import bg_hom3d

# Load data
#------------------------
# Ground truth
data = np.genfromtxt("./data/figure4_Pdata.txt", dtype=None, skip_header=1)
r,Ptruth = [data[:,i] for i in range(data.shape[1])]

# Input signal
data = np.genfromtxt("./data/figure4_Vdata.txt", dtype=None, skip_header=1)
t1,V1,B1,t2,V2,B2,t3,V3,B3,t4,V4,B4 = [data[:,i] for i in range(data.shape[1])]

ts = [t1,t2,t3,t4]
Vs = [V1,V2,V3,V4]
Bs = [B1,B2,B3,B4]

for i in range(len(Vs)):
        
    V = Vs[i]
    t = ts[i]
    B = Bs[i]
    # One-step analysis
    #--------------------------------------------------------------------------
    # Fit the background and distribution simultaneously
    fit = dl.fitsignal(V,t,r,'P',bg_hom3d,ex_4pdeer)
    Vfit = fit.V
    Pfit = fit.P
    Bfit = fit.B
    lam = fit.exparam
    Bfit = (1-lam)*Bfit    

    # Two-step analysis
    #--------------------------------------------------------------------------
    
    # Fit the background...
    tstart = [3.7, 1.6, 0.9, 0.4][i] # background fit start, in us
    mask = t>tstart
    def Bmodel(par):
        lam,conc = par # unpack parameters
        B = (1-lam)*dl.bg_hom3d(t[mask],conc,lam)
        return B
    #       lam    conc
    par0 = [0.3,   50]
    lb   = [0.01,  0.01]
    ub   = [1,     1000]
    fit = dl.fitparamodel(V[mask],Bmodel,par0,lb,ub,rescale=False)
    lam2,conc2 = fit.param
    Bfit2 = dl.bg_hom3d(t,conc2,lam2)

    # ...and then the distribution
    KBfit = dl.dipolarkernel(t,r,lam2,Bfit2)
    fit = dl.fitregmodel(V,KBfit,r,'tikhonov','aic')
    Pfit2 = fit.P
    Bfit2 = (1-lam2)*Bfit2
    Vfit2 = KBfit@Pfit2
    
    # Plot the results
    #--------------------------------------------------------------------------
    plt.subplot(3,4,i+1)
    plt.plot(t,V,'k.',t,Vfit,'b-',t,Bfit,'b--',t,(1-0.3)*B,'k--',linewidth=1.5)
    plt.xlabel('t [$\mu s$]')
    plt.ylabel('V(t)')

    plt.subplot(3,4,i+5)
    plt.plot(t,V,'k.',t,Vfit2,'r-',t,Bfit2,'r--',t,(1-0.3)*B,'k--',linewidth=1.5)
    plt.xlabel('t [$\mu s$]')
    plt.ylabel('V(t)')
        
    plt.subplot(3,4,i+9)
    plt.plot(r,Ptruth,'k',r,Pfit,'b',r,Pfit2,'r',linewidth=1.5)
    plt.xlim([3.5, 6.5])
    plt.xlabel('r [nm]')
    plt.ylabel('P(r) [nm$^{-1}$]')

plt.tight_layout()
# %%
