%--------------------------------------------------------------------------
% Figure 7(a) - Fitting a 4-pulse DEER signal with "2+1 arifact"
%--------------------------------------------------------------------------
% This DeerLab script was used to make Fig.7a of the paper: 
%
% Luis Fabregas Ibanez, Gunnar Jeschke, Stefan Stoll 
% DeerLab: A comprehensive toolbox for analyzing dipolar EPR spectroscopy data
% Magnetic Resonance, 2020
%
% DeerLab version: 
%   Release: 0.9.0
%--------------------------------------------------------------------------

clear,clc,clf

rng(1)

% Load example data
[r,P] = groundtruth('figure7a');

%Construct time-axis
t = linspace(-0.1,4.5,500);

% Multipathway parameters
Ts2 = max(t) - 0.005;
lambdas = [0.5 0.25 0.1];
pathways = ex_ovl4pdeer([lambdas Ts2]);

% Background parameters
kappa = 0.25;
strfact = 0.93;
Bmodel = @(t,lam) bg_strexp(t,[kappa strfact],lam);

%Genrate signal
K = dipolarkernel(t,r,pathways,Bmodel);
B = dipolarbackground(t,pathways,Bmodel);
V = K*P + whitegaussnoise(t,0.004);

% Fit the 4pDEER signal with the "2+1 artifact"
[Vfit,Pfit,Bfit] = fitsignal(V,t,r,'P',@bg_strexp,@ex_ovl4pdeer);

% Plot
subplot(221)
plot(t,V,'k.',t,Vfit,'r',t,B,'k--',t,Bfit,'r--','LineWidth',1.5)
axis tight, grid on, box on
set(gca,'fontsize',14)
xlabel('t [\mus]'),ylabel('V(t)')
legend('V_{exp}','V_{fit}','B_{truth}','B_{fit}','Location','eastout')

subplot(222)
plot(r,P,'k',r,Pfit,'r','LineWidth',1.5)
axis tight
set(gca,'fontsize',14)
xlabel('r [nm]'),ylabel('P(r) [nm^{-1}]')
legend('P_{truth}','P_{fit}','Location','eastout')
xlim([1.5 5])

%--------------------------------------------------------------------------
% Figure 7(b) - Fitting a 5-pulse DEER signal
%--------------------------------------------------------------------------
% This DeerLab script was used to make Fig.7b of the paper: 
%
% Luis Fabregas Ibanez, Gunnar Jeschke, Stefan Stoll 
% DeerLab: A comprehensive toolbox for analyzing dipolar EPR spectroscopy data
% Magnetic Resonance, 2020
%
% DeerLab version: 
%   Release: 0.9.0
%--------------------------------------------------------------------------

% Fix random numbers for reproduceable results
rng(1)

% Load example data
[r,P] = groundtruth('figure7b');

% Define time-axis
t = linspace(-0.1,9,500);

% Multipathway parameters
Ts2 = 4.6;
lambdas = [0.65 0.75 0.25];
pathways = ex_5pdeer([lambdas Ts2]);

% Background parameters
kappa = 0.1;
strfact = 0.88;
Bmodel = @(t,lam) bg_strexp(t,[kappa strfact],lam);

% Simulate the 5pDEER dipolar signal
K = dipolarkernel(t,r,pathways,Bmodel);
B = dipolarbackground(t,pathways,Bmodel);
V = K*P + whitegaussnoise(t,0.005);

% Fit the 5pDEER signal
[Vfit,Pfit,Bfit] = fitsignal(V,t,r,'P',@bg_strexp,@ex_5pdeer);

% Plot
subplot(223)
plot(t,V,'k.',t,Vfit,'b',t,B,'k--',t,Bfit,'b--','LineWidth',1.5)
axis tight,box on, grid on
xlabel('t [\mus]'),ylabel('V(t)')
set(gca,'fontsize',14)
legend('V_{exp}','V_{fit}','B_{truth}','B_{fit}','Location','eastout')

subplot(224)
plot(r,P,'k',r,Pfit,'b','LineWidth',1.5)
axis tight,box on, grid on
xlabel('r [nm]'),ylabel('P(r) [nm^{-1}]')
set(gca,'fontsize',14)
xlim([2.5 5])
legend('P_{truth}','P_{fit}','Location','eastout')
