# %%
"""
--------------------------------------------------------------------------
Figure 6a - Fitting a 4-pulse DEER signal with "2+1 arifact"
-------------------------------------------------------------------------
This DeerLab script was used to make Fig.6a of the paper: 

Luis Fabregas Ibanez, Gunnar Jeschke, Stefan Stoll 
DeerLab: A comprehensive toolbox for analyzing dipolar EPR spectroscopy data
Magnetic Resonance, 2020

DeerLab version: 
Release: 0.10.0
--------------------------------------------------------------------------
"""

import numpy as np
import matplotlib.pyplot as plt
import deerlab as dl
from deerlab.ex_models import ex_ovl4pdeer
from deerlab.bg_models import bg_strexp

# Load data
#------------------------
# Ground truth
data = np.genfromtxt("./data/figure6a_Pdata.txt", dtype=None, skip_header=1)
r,Ptruth = [data[:,i] for i in range(2)]

# Input signal
data = np.genfromtxt("./data/figure6b_Vdata.txt", dtype=None, skip_header=1)
t,V = [data[:,i] for i in range(2)]

# Fit the 4pDEER signal with the "2+1 artifact"
fit = dl.fitsignal(V,t,r,'P',bg_strexp,ex_ovl4pdeer,uqanalysis=False)
Vfit = fit.V 
Pfit = fit.P 
Bfit = fit.B

# Plot
plt.subplot(121)
plt.plot(t,V,'k.',t,Vfit,'b',t,Bfit,'b--', linewidth=1.5)
plt.xlabel('t [$\mu s$]')
plt.ylabel('V(t)')
plt.legend(['V_{exp}','V_{fit}','B_{truth}','B_{fit}'])

plt.subplot(122)
plt.fill(r,Ptruth,color='gray',alpha=0.4)
plt.plot(r,Pfit,'b', linewidth=1.5)
plt.xlabel('r [nm]')
plt.ylabel('P(r) [nm$^{-1}$]')
plt.legend(['P$_{truth}$','P$_{fit}$'])
plt.xlim([1.5, 5])

# %%
"""
--------------------------------------------------------------------------
Figure 6b - Fitting a 5-pulse DEER signal
--------------------------------------------------------------------------
This DeerLab script was used to make Fig.6b of the paper: 

Luis Fabregas Ibanez, Gunnar Jeschke, Stefan Stoll 
DeerLab: A comprehensive toolbox for analyzing dipolar EPR spectroscopy data
Magnetic Resonance, 2020

DeerLab version: 
Release: 0.10.0
--------------------------------------------------------------------------
"""

import numpy as np
import matplotlib.pyplot as plt
import deerlab as dl
from deerlab.ex_models import ex_5pdeer
from deerlab.bg_models import bg_strexp

# Load data
#------------------------
# Ground truth
data = np.genfromtxt("./data/figure6b_Pdata.txt", dtype=None, skip_header=1)
r,Ptruth = [data[:,i] for i in range(2)]

# Input signal
data = np.genfromtxt("./data/figure6b_Vdata.txt", dtype=None, skip_header=1)
t,V = [data[:,i] for i in range(2)]

# Fit the 5pDEER signal
fit = dl.fitsignal(V,t,r,'P',bg_strexp,ex_5pdeer,uqanalysis=False)
Vfit = fit.V 
Pfit = fit.P 
Bfit = fit.B

# Plot
plt.subplot(121)
plt.plot(t,V,'k.',t,Vfit,'b',t,Bfit,'b--', linewidth=1.5)
plt.xlabel('t [$\mu s$]')
plt.ylabel('V(t)')
plt.legend(['V_{exp}','V_{fit}','B_{truth}','B_{fit}'])

plt.subplot(122)
plt.fill(r,Ptruth,color='gray',alpha=0.4)
plt.plot(r,Pfit,'b', linewidth=1.5)
plt.xlabel('r [nm]')
plt.ylabel('P(r) [nm$^{-1}$]')
plt.legend(['P$_{truth}$','P$_{fit}$'])
plt.xlim([2.5, 5])



# %%
