# %%
"""
--------------------------------------------------------------------------
Figure 7 - Global fit of signals from different experiments
--------------------------------------------------------------------------
This DeerLab script was used to make Fig.7 of the paper: 

Luis Fabregas Ibanez, Gunnar Jeschke, Stefan Stoll 
DeerLab: A comprehensive toolbox for analyzing dipolar EPR spectroscopy data
Magnetic Resonance, 2020

DeerLab version: 
Release: 0.10.0
--------------------------------------------------------------------------
"""

import numpy as np
import matplotlib.pyplot as plt
import deerlab as dl
from deerlab.ex_models import ex_4pdeer, ex_5pdeer
from deerlab.bg_models import bg_exp

# Load data
#------------------------
# Ground truth
data = np.genfromtxt("./data/figure7_Pdata.txt", dtype=None, skip_header=1)
r,Ptruth = [data[:,i] for i in range(2)]

# Input signal
data = np.genfromtxt("./data/figure7_Vdata.txt", dtype=None, skip_header=1)
t4p,V4p,t5p,V5p = [data[:,i] for i in range(4)]

# Parameter boundaries for the experiment models:
#         ex_4p      ex_5p
#         ----- ------------------
#          lam  Lam0 lam1 lam2 T02    
ex_ub   = [ 1,   1,   1,   1,   4]
ex_lb   = [ 0,   0,   0,   0,   2]
ex_par0 = [0.5, 0.5, 0.5, 0.5,  3]

# Run global fit
fit = dl.fitsignal([V4p, V5p],[t4p, t5p],r,'P',bg_exp,[ex_4pdeer,ex_5pdeer],
                        par0=[None,None,ex_par0],ub=[None,None,ex_ub],lb=[None,None,ex_lb])
# Get the results
V4pfit = fit.V[0]
V5pfit = fit.V[1]
Pfit = fit.P

# Plot results
plt.subplot(121)
plt.plot(t4p,V4p,'k.',t4p,V4pfit)
plt.plot(t5p,V5p + 0.5,'k.',t5p,V5pfit + 0.5)
plt.xlabel('t [$\mu s$]')
plt.ylabel('V(t)')
plt.legend(['4pDEER data','4pDEER fit','5pDEER data','5pDEER fit'])

plt.subplot(122)
plt.fill(r,Ptruth,color='gray',alpha=0.4)
plt.plot(r,Pfit,'b')
plt.xlabel('r [nm]')
plt.ylabel('P(r) [nm$^{-1}$]')
plt.legend(['global fit','truth'])




# %%
