%--------------------------------------------------------------------------
% Figure 11 - Titration curve with parameter-free distributions
%--------------------------------------------------------------------------
% This DeerLab script was used to make Fig.11 of the paper: 
%
% Luis Fabregas Ibanez, Gunnar Jeschke, Stefan Stoll 
% DeerLab: A comprehensive toolbox for analyzing dipolar EPR spectroscopy data
% Magnetic Resonance, 2020
%
% DeerLab version: 
%   Release: 0.8.beta
%   Commit: e875bbc3ff613cf26896d9aa42ecea64069b6656
%--------------------------------------------------------------------------

clc, clear,clf

%Fix random numbers for reproduceable results
rng(2)

%Prepare equilibrium of type:
%   A + L <-> B
KD = 5.65;  % dissociation constant, uM
Ctot = 1; % total protein concentration, uM
L = [0.3 1 3 10 30 100 300]; % total ligand concentration, uM
nDataSets = numel(L);

% Calculate molar fractions
%    a (for A = protein without ligand)
%    b (for B = protein with bound protein)
Kb = 1/KD;
for q = 1:nDataSets
    xbound_ = roots([Kb*Ctot -(Kb*L(q) + Kb*Ctot + 1) Kb*L(q)]);
    xbound(q) = xbound_(xbound_<=1 & xbound_>=0);
end

%Generate ground truth distance distributions for each fraction
[rtruth,P1] = groundtruth('figure10_1');
[~,P2] = groundtruth('figure10_2');

% Generate different dipolar time-axes
tmax = 1 + randi(6,nDataSets,1);
for i=1:length(tmax)
    ts{i} = linspace(-0.1,tmax(i),300);
end

% Generate different background decays, noise levels, and modulation depths
% for each dataset
ks = linspace(0.2,1.15,nDataSets)*0.25;
ks = ks(randperm(nDataSets));
noiselevels = 0.005 + 0.02*rand(nDataSets,1);
lams = linspace(0.75,1.15,nDataSets)*0.45;
lams = lams(randperm(nDataSets));

%Generate set of dipolar signals
for i = 1:nDataSets
    
    % Prepare the sum distance distribution from corresponding molar fractions
    P = xbound(i)*P1  + (1 - xbound(i))*P2;
    Ps{i} = P;
    %Prepare corresponding dipolar kernel
    K = dipolarkernel(ts{i},rtruth,lams(i),bg_exp(ts{i},ks(i)));
    %Simulate dipolar signal
    Vs{i} = K*Ps{i} + whitegaussnoise(ts{i},noiselevels(i),'rescale');

end

%Define the fit distance axis
r = linspace(2,6,300);

% Define start values & boundaries for fit parameters
% param0 =  [b ks lams];
param0 =  [linspace(0,1,nDataSets) 0.25*ones(1,nDataSets) 0.25*ones(1,nDataSets)];
lower  =  [zeros(1,nDataSets) 0*ones(1,nDataSets) 0*ones(1,nDataSets)];
upper  =  [ones(1,nDataSets) 1*ones(1,nDataSets) 1*ones(1,nDataSets)];

% Gobal fit of the parametric time-domain model with parameter-free distributions
parfit = fitparamodel(Vs,@(t,p,idx)mymodel(t,p,idx,ts,r,Vs,Ps,rtruth,nDataSets),ts,param0,'Lower',lower,'Upper',upper,'tolfun',1e-8,'globalweights',ones(nDataSets,1));

%Get the fitted molar fractions
xbound_fit = parfit(1:nDataSets);
xunbound_fit = 1 - xbound_fit;

%Control label switching
if mean(diff(xbound_fit))<0
    tmp = xunbound_fit;
    xunbound_fit = xbound_fit;
    xbound_fit = tmp;
end

% Re-calculate time and distance-domain fits
for i=1:nDataSets
    [Vfit{i},Pfit{i},B{i}] = mymodel(ts{i},parfit,i,ts,r,Vs,Ps,rtruth,nDataSets);
    B{i} = (1 - parfit(2*nDataSets + i))*B{i};
end

%Extrapolate the molar fraction values for plotting
Lfine = logspace(min(log10(L)),max(log10(L)),50);
for q = 1:numel(Lfine)
    p = [Kb*Ctot -(Kb*Lfine(q)+Kb*Ctot+1) Kb*Lfine(q)];
    xbound_ = roots(p);
    xbound(q) = xbound_(xbound_<=1 & xbound_>=0);
end
xunbound = 1-xbound;

%Plot input data & time-domain fits
figure(5),clf,hold on
for j=1:length(ts)
    i = length(ts)+1-j;
    plot(ts{i},Vs{i}+0.5*(j-1),'k.',ts{i},Vfit{i}+0.5*(j-1),'r',ts{i},B{i}+0.5*(j-1),'b--','LineWidth',1.5,'MarkerSize',8)
    text(4,0.5*j,sprintf('%.2f \\muM',L(i)))
end
axis tight,box on
xlabel('Time [\mus]'),ylabel('V(t)')
set(gca,'FontSize',15,'yticklabel',[])

%Plot ground truth distributions & distance-domain fits
r1 = r(r<8);
r2 = r(r>=1);
rjoined = [r1 r2];
figure(4),clf
for i=1:length(Pfit)
    subplot(1,8,i)
    tmp = Pfit{i};
    P1 = tmp(1:numel(r1));
    P2 = tmp(numel(r1)+1:end);
    plot(r1,P1/mean(diff(r2)),'r',r2,P2/mean(diff(r2)),'b',rtruth,Ps{i}/sum(Ps{i})/mean(diff(rtruth)),r1,P2/mean(diff(r2))+P1/mean(diff(r2)),'LineWidth',1.5)
    axis tight,box on
    xlim([2 6])
    title(sprintf('c_{ligand}=%.2f \\muM',L(i)))
end
legend('bound','unbound','truth','sum')
xlabel('distance [nm]'),ylabel('P(r)')
set(gca,'FontSize',15,'yticklabel',[])

%Plot titration curve with ground truth and fitted points (log scale)
figure(7),clf
plot(log10(L),xbound_fit,'r.',log10(L),xunbound_fit,'b.',log10(Lfine),xbound,'r',log10(Lfine),xunbound,'b','LineWidth',1.5,'MarkerSize',25)
axis tight,box on
xlabel('log(Ligand conc.)'),ylabel('Molar fraction')
set(gca,'fontsize',15,'ytick',0:0.2:1,'yticklabel',0:0.2:1)
legend('fit x_{bound}','fit x_{unbound}','truth x_{bound}','truth x_{unbound}')
ylim([0 1])
ytickformat('%.1f')

%Function definition of the global fit with parameter-free distributions
function [Vfit,Pfit,B] = mymodel(t,p,idx,ts,r,Vs,Ps,rtrue,nDataSets)

persistent Ks Pjoined
% Make the dipolar kernels and fitted distributions as persistent variables
% since they do not need to be calculated for each signal again (since the
% fit is global). Speed enhancement = number of globally fitted signals 

% Extract parameters
k = p(nDataSets+1:2*nDataSets);
lam = p(2*nDataSets+1:end);

% Define fit axis for first component...
r1 = r(r < 8);
% ...and second component..
r2 = r(r >= 1);
% ...and concatenate them into a single vector
rjoined = [r1 r2];

% Run this section only for the first signal, skip for the rest
if idx==1
    
    % Loop over all signals being globally fitted
    for i=1:numel(Vs)
        
        % Prepare the dipolar kernels for both components
        K1 = dipolarkernel(ts{i},r1);
        K2 = dipolarkernel(ts{i},r2);
        % Concatenate them in the distance-dimension
        Kjoined = [K1 K2];
        
        % Normalize the time-dimension
        [~,t0] = min(abs(ts{i}));
        Kjoined = Kjoined./Kjoined(t0,:);
        
        % Introduce modulation depth and background information
        Kjoined = (1-lam(i) + lam(i)*Kjoined).*bg_exp(ts{i},k(i));
        
        % Weight the distance-dimension according to the molar fractions
        amps = [ones(1,numel(r1))*p(i) ones(1,numel(r1))*(1 - p(i))];
        Kjoined = amps.*Kjoined;
        
        % Save the joined dipolar kernel for the current signal
        Ks{i} = Kjoined;
    end
    % Global Tikhonov regularization using the joined dipolar kernels
    rng(2)
    alpha = 25;
    Pjoined = fitregmodel(Vs,Ks,rjoined,'tikh',alpha);
    % Normalize integral
    Pjoined = Pjoined/sum(Pjoined);
end

% Extract the fits for each of the components from the joined vector
Pfit2 = Pjoined(end-numel(r2)+1:end);
Pfit1 = Pjoined(1:numel(r1));
% Normalize their integrals
Pfit1 = Pfit1/sum(Pfit1);
Pfit2 = Pfit2/sum(Pfit2);

% Compute the time-domain fit of the current dipolar signal
Vfit = Ks{idx}*[Pfit1; Pfit2];

% Return the fits scaled by their respective molar fractions
Pfit = [p(idx)*Pfit1; (1-p(idx))*Pfit2];

% Compute the fitted background to return as output
B = bg_exp(ts{idx},k(idx));

% Plot intermediate results
subplot(121),hold on
if idx==1
    cla
end
plot(t,Vs{idx}+idx/2,'k.',t,Vfit+idx/2,'r')
ylim([0.5 5.5])
axis tight, grid on, box on
xlabel('t [\mus]')
ylabel('V(t)')
legend('data','fit')
subplot(122)
Ptruth = Ps{idx}/sum(Ps{idx})/mean(diff(rtrue));
Pfrac1 = p(idx)*Pfit1/mean(diff(r1));
Pfrac2 = (1-p(idx))*Pfit2/mean(diff(r2));
plot(rtrue,Ptruth,r1,Pfrac1,r2,Pfrac2)
axis tight, grid on, box on
xlabel('r [nm]')
ylabel('P(r) [nm^{-1}]')
legend('truth (sum)','fraction 1','fraction 2')
drawnow

end