# %%
"""
--------------------------------------------------------------------------
Figure 8 - Titration curve with parameter-free distributions
--------------------------------------------------------------------------
This DeerLab script was used to make Fig.8 of the paper: 

Luis Fabregas Ibanez, Gunnar Jeschke, Stefan Stoll 
DeerLab: A comprehensive toolbox for analyzing dipolar EPR spectroscopy data
Magnetic Resonance, 2020

DeerLab version: 
Release: 0.10.0
#--------------------------------------------------------------------------
"""

import numpy as np
import matplotlib.pyplot as plt
import deerlab as dl
from deerlab.bg_models import bg_hom3d


# Load data
#------------------------
# Ground truth
data = np.genfromtxt("./data/figure8_Pdata.txt", dtype=None, skip_header=1)
rtruth,P1,P2,P3,P4,P5,P6,P7 = [data[:,i] for i in range(data.shape[1])]

# Input signal
data = np.genfromtxt("./data/figure8_Vdata.txt", dtype=None, skip_header=1)
t1,V1,t2,V2,t3,V3,t4,V4,t5,V5,t6,V6,t7,V7 = [data[:,i] for i in range(data.shape[1])]

# Collect into lists
ts = [t1,t2,t3,t4,t5,t6,t7]
Vs = [V1,V2,V3,V4,V5,V6,V7]
Ps = [P1,P2,P3,P4,P5,P6,P7]
nDatasets = len(Vs)

# Distance axes for states A and B
rA = np.linspace(2,6,300)
rB = np.linspace(2,6,300)

# Model definitions
#------------------------
# In this example we will simulate a protein system in their states A (natural) 
# and B (changed upon addition of a ligand L) given by the chemical equilibrium A + L <-> B.
def chemicalequilibrium(Kdis,L):
    Ctot = 1 # total protein concentration, uM

    # # Get fraction of state B
    Kb = 1/Kdis
    xB = np.zeros_like(L)
    for q in range(len(L)):
        xB_ = np.roots([Kb*Ctot, -(Kb*L[q] + Kb*Ctot + 1), Kb*L[q]])
        try:
            xB[q] = xB_[(xB_<=1) & (xB_>=0)]
        except:
            xB[q] = np.minimum(1,np.maximum(0,xB_[0]))
    # Get fraction of state A
    xA = 1 - xB

    return xA,xB

# Next, we define the dipolar kernel model as the non-linear function of the
# SNLLS problem. This function needs to take the parameters and return a
# cell-array of kernels, each one for the corresponding datasets that we
# have. 
# Since we have a total distribution of the form 
#     ``P = xA*PA + xB*PB``
# we can define an augmented kernel as
#     ``K = [xA*KA xB*KB]``
# such that 
#     ``K@[PA PB] = V``
# and the vector ``[PA PB]`` constitutes the linear part fitted by SNLLS.
def Kmodel(par):

    Nsignals = len(ts)

    # Unpack parameters
    lams = par[0:nDatasets]
    concs = par[nDatasets:2*nDatasets]
    xBs = par[2*nDatasets:3*nDatasets]

    Ks = [[]]*Nsignals
    # General the dipolar kernels
    for i in range(Nsignals):
        B = bg_hom3d(ts[i],concs[i],lams[i])
        # Kernel for fraction A
        KstateA = dl.dipolarkernel(ts[i],rA,lams[i],B)
        # Kernel for fraction B
        KstateB = dl.dipolarkernel(ts[i],rB,lams[i],B)
        xA = np.maximum(1-xBs[i],0)
        Ks[i] = np.concatenate((xA*KstateA, xBs[i]*KstateB),axis=1)

    return Ks

# Chemical ground truth
#------------------------
# Since we know the true Kdis we can simulate the true populations of states A and B
L = [0.3, 1, 3, 10, 30, 100, 300] # total ligand concentration, uM
Kdis = 5.65  # dissociation constant, uM
# Populations of states A and B
[xA,xB] = chemicalequilibrium(Kdis,L)

# %%
# Psuedotitration SNLLS Analysis
# ------------------------------

# Non-linear parameters:
par0_lam = [0.25]*nDatasets  # modulation depths start values
ub_lam = [0.6]*nDatasets    # modulation depths upper bounds
lb_lam = [0.0]*nDatasets    # modulation depths lower bounds

par0_k = [50]*nDatasets    # decay rates start values
ub_k = [2000]*nDatasets      # decay rates upper bounds
lb_k = [0.05]*nDatasets       # decay rates lower bounds

par0_xB = [0.0,0.2,0.4,0.5,0.6,0.8,1.0] # state B populations start values    
ub_xB = [1.0]*nDatasets     # decay rates start values
lb_xB = [0.0]*nDatasets     # decay rates start values

par0 = par0_lam + par0_k + par0_xB # start values
lb = lb_lam + lb_k + lb_xB # start values
ub = ub_lam + ub_k + ub_xB # start values

# Linear parameters:
#     |-------PA--------||--------PB--------|
lbl = np.concatenate((np.zeros_like(rA), np.zeros_like(rB))) # Non-negativity constraint
ubl = [] # Unconstrained

# Run SNLLS optimization
fit = dl.snlls(Vs,Kmodel,par0,lb,ub,lbl,ubl,uqanalysis=False)
# Extract fit results
parfit = fit.nonlin
Pfit = fit.lin

# %%
plt.figure(figsize=(12,12))

# Simulate fits
Ksfit = Kmodel(parfit)
Vsfit = []
plt.subplot(3,2,(1,3))
for i in range(nDatasets):
    Vsfit.append(Ksfit[i]@Pfit)
    plt.plot(ts[i],Vs[i]+i/2,'k.',ts[i],Vsfit[i]+i/2,'r',linewidth=1.5)
plt.grid(alpha =0.3)
plt.xlabel('t [$\mu s$]')
plt.ylabel('V(t) [a.u.]')
plt.legend(['data','fit'])

xBfit = parfit[2*nDatasets:3*nDatasets]
xAfit = 1 - xBfit
plt.subplot(2,2,(2,4))
for i in range(nDatasets):
    PAfit = xAfit[i]*Pfit[0:len(rA)]
    PBfit = xBfit[i]*Pfit[len(rA):len(rB)+len(rA)]
    plt.plot(rA,PAfit+PBfit + 2*i,'k',rA,PAfit+2*i,'r',rB,PBfit+2*i,'b',linewidth=1.5)

plt.grid(alpha =0.3)
plt.xlabel('r [nm]')
plt.ylabel('P(r)')
plt.legend(['Pfit','state A','state B'])
plt.xlim([2,7])

plt.subplot(325)
plt.plot(np.log10(L),xA,'r-',np.log10(L),xB,'b-')
plt.plot(np.log10(L),xAfit,'ro',np.log10(L),xBfit,'bo',linewidth=1.5)
plt.grid(alpha =0.3)
plt.xlabel('log$_{10}$([L])')
plt.ylabel('Fractions')
plt.legend(['state A','state B'])
plt.ylim([0,1])

# %%
