%--------------------------------------------------------------------------
% Figure 11 - Uncertainty estimation methods
%--------------------------------------------------------------------------
% This DeerLab script was used to make Fig.11 of the paper: 
%
% Luis Fabregas Ibanez, Gunnar Jeschke, Stefan Stoll 
% DeerLab: A comprehensive toolbox for analyzing dipolar EPR spectroscopy data
% Magnetic Resonance, 2020
%
% DeerLab version: 
%   Release: 0.9.0
%--------------------------------------------------------------------------

clear,clc,clf

% Fix random numbers for reproduceable results
rng(1)

% Load data for the examples
[r,P] = groundtruth('figure11');

% Construct time-axis
t = linspace(-0.2,4,300);
% Background parameters
kappa = 0.25;
lambda = 0.35;
B = bg_exp(t,lambda*kappa);
% Construct dipolar kernel
KB = dipolarkernel(t,r,lambda,B);
% Noise standard deviation
sig = 0.02;
% Generate dipolar signal
V = KB*P + whitegaussnoise(t,sig);

% Fit the signal
[Vfit,Pfit,Bfit,parfit,modelci,paramci,stat] = fitsignal(V,t,r,'P',@bg_exp,@ex_4pdeer,[],'RegParam',0.5);

%Get confidence intervals on Pfit
Pci = modelci.Pfit;

%Plot signal and fit
subplot(231)
hold on
plot(t,V,'k.',t,Vfit,'m','Linewidth',1.5)  
axis tight, box on
xlabel('t [\mus]'),ylabel('V(t)')
legend('V_{exp}','V_{fit}','Location','eastout')
text(2.5,0.9,sprintf('\\chi^2 = %.3f',stat.chi2red))
paramci = [paramci.ex; paramci.bg];
parfit = [parfit.ex; parfit.bg];

% Generate covariance-based distributions of the fit parameters
%-------------------------------------------------------------------

% Student's t-distribution critical value (95%)
z = 1.968;
paramsig = abs(paramci(:,1) - parfit)/z;

%Gaussian distribution for modulation depth
parVals1 = linspace(0.15,0.55,300);
P1 = exp(-(parfit(1) - parVals1).^2./(2*paramsig(1)^2));
%Gaussian distribution for decay rate
parVals2 = linspace(0.05,0.5,300);
P2 = exp(-(parfit(2) - parVals2).^2./(2*paramsig(2)^2));


% Generate distribution of residuals
%-------------------------------------------------------------------

%Compute fit residuals
residuals = (V - Vfit)/std(V - Vfit);

% Standard Gaussian distribution
resVals = linspace(-3,3,200);
stdGauss = exp(-resVals.^2/2);

%Fit the Gaussian to the histogram of residuals (for display)
yhist = histcounts(residuals,'normalization','probability');
p = fminsearch(@(p)norm(yhist - p*stdGauss.'),1);
stdGauss = stdGauss*max(yhist);

% Plot histogram of residuals
subplot(234),cla,hold on
histogram(residuals,'normalization','probability','FaceColor','w')
plot(resVals,stdGauss,'m','linewidth',2)
axis tight, box on
legend('Histogram','Std. Gaussian','Location','eastout')


% Bootstrap analysis
%-------------------------------------------------------------------

fcn = @(V) bootfcn(V,r,t);
[bootci,stats] = bootan(fcn,V,Vfit,1000,'verbose',true);


% Plots
%-------------------------------------------------------------------

subplot(232)
cla, hold on
plot(r,P,'k',r,Pfit,'r')
fill([r fliplr(r)],[Pci{1}(:,1); flipud(Pci{1}(:,2))],'r','facealpha',0.2,'linestyle','none')
fill([r fliplr(r)],[Pci{2}(:,1); flipud(Pci{2}(:,2))],'r','facealpha',0.4,'linestyle','none')
axis tight, box on
xlabel('r [nm]'),ylabel('P(r) [nm^{-1}]')
legend('P_{truth}','P_{fit}','95%-CI','50%-CI','Location','eastout')
ylim([0 1.5])

subplot(235)
cla,hold on
Pbootci = bootci{2};
plot(r,P,'k',r,Pfit,'b')
fill([r fliplr(r)],[Pbootci.ci95(:,1); flipud(Pbootci.ci95(:,2))],'b','facealpha',0.2,'linestyle','none')
fill([r fliplr(r)],[Pbootci.ci50(:,1); flipud(Pbootci.ci50(:,2))],'b','facealpha',0.4,'linestyle','none')
axis tight, grid off, box on
xlabel('r [nm]'),ylabel('P(r) [nm^{-1}]')
legend('P_{truth}','P_{fit}','95%-CI','50%-CI','Location','eastout')
ylim([0 1.5])

subplot(233)
paramboot = stats{1};
hold on
normfact = max(paramboot(1).boothist.bins);
histogram('BinCounts',paramboot(1).boothist.bins(1:2:end-1), 'BinEdges', paramboot(1).boothist.edges(1:2:end),'FaceColor','w');
plot(parVals1,P1*normfact,'r','Linewidth',1.5)
plot(paramboot(1).bootdist.values, paramboot(1).bootdist.pdf/max(paramboot(1).bootdist.pdf).*normfact,'b','Linewidth',1.5)   
axis tight, grid off, box on
xlabel('\lambda')
ylabel('Probability')
legend('Boot. Histogram','Standard Distribution','Boot. Distribution','Location','eastout')
xlim([0.3 0.4])

subplot(236)
paramboot = stats{1};
hold on
normfact = max(paramboot(2).boothist.bins);
histogram('BinCounts',paramboot(2).boothist.bins(1:2:end-1), 'BinEdges', paramboot(2).boothist.edges(1:2:end),'FaceColor','w');
plot(parVals2,P2*normfact,'r','Linewidth',1.5)
plot(paramboot(2).bootdist.values, paramboot(2).bootdist.pdf/max(paramboot(2).bootdist.pdf).*normfact,'b','Linewidth',1.5)   
axis tight, grid off, box on
xlabel('\kappa [\mus^{-1}]')
ylabel('Probability')
legend('Boot. Histogram','Standard Distribution','Boot. Distribution','Location','eastout')
xlim([0.15 0.35])

%Bootstrapped function
function [parfit,Pfit,Vfit] = bootfcn(V,r,t)
    [Vfit,Pfit,~,parfit] = fitsignal(V,t,r,'P',@bg_exp,@ex_4pdeer,[],'RegParam',0.5);
    parfit = [parfit.ex; parfit.bg];
end