# %%
"""
--------------------------------------------------------------------------
Figure 10 - Uncertainty estimation methods
--------------------------------------------------------------------------
This DeerLab script was used to make Fig.10 of the paper: 

Luis Fabregas Ibanez, Gunnar Jeschke, Stefan Stoll 
DeerLab: A comprehensive toolbox for analyzing dipolar EPR spectroscopy data
Magnetic Resonance, 2020

DeerLab version: 
Release: 0.10.0
--------------------------------------------------------------------------
""" 

import numpy as np
import matplotlib.pyplot as plt
import deerlab as dl
from deerlab.ex_models import ex_4pdeer
from deerlab.bg_models import bg_hom3d

# Load data
#-------------------------
# Ground truth
data = np.genfromtxt("./data/figure10_Pdata.txt", dtype=None, skip_header=1)
r,Ptruth = [data[:,i] for i in range(2)]

# Input signal
data = np.genfromtxt("./data/figure10_Vdata.txt", dtype=None, skip_header=1)
t,V = [data[:,i] for i in range(2)]

# Fit the signal
fit = dl.fitsignal(V,t,r,'P',bg_hom3d,ex_4pdeer,regparam=0.5)
# Note: in this example we use a fixed regularization parameter for the sake,
# of speed of this demonstration. In practice it is recommended the use of a 
# selection criterion such as AIC
Vfit = fit.V
Pfit = fit.P
stats = fit.stats
paramuq = [fit.exparamUncert, fit.bgparamUncert] 
parfit = [fit.exparam, fit.bgparam]

#Get confidence intervals on Pfit
Puq = fit.Puncert
Pci95 = Puq.ci(95)
Pci50 = Puq.ci(50)

# Print fit chi2red
print('chi2red = {:.3f}'.format(stats['chi2red']))

# %%
# Generate distribution of residuals
#-------------------------------------------------------------------

#Compute fit residuals
residuals = (V - Vfit)/np.std(V - Vfit)

# Standard Gaussian distribution
resVals = np.linspace(-3,3,200)
stdGauss = np.exp(-resVals**2/2)
stdGauss = stdGauss/np.trapz(stdGauss,resVals)

# %%
# Generate covariance-based distributions of the fit parameters
#-------------------------------------------------------------------

#Gaussian distribution for modulation depth
parVals1,P1 = paramuq[0].pardist(0)
#Gaussian distribution for decay rate
parVals2,P2 = paramuq[1].pardist(0)


# %%
# Bootstrap analysis
#-------------------------------------------------------------------

#Bootstrapped function
def bootfcn(V):
    fit = dl.fitsignal(V,t,r,'P',bg_hom3d,ex_4pdeer,regparam=0.5)
    return fit.exparam, fit.bgparam, fit.P, fit.V

bootuq = dl.bootan(bootfcn,V,Vfit,1000,verbose=True)

# Bootstrapped confidence intervals of Pfit
Pbootuq = bootuq[2]
Pbootci95 = Pbootuq.ci(95)
Pbootci50 = Pbootuq.ci(50)
# %%
# Bootstrapped distribution of modulation depth values
lamboot = bootuq[0]
values1,pdf1 = lamboot.pardist(0)

# Bootstrapped distribution of spin concentrations
concboot = bootuq[1]
values2,pdf2 = concboot.pardist(0)

# %%

# Plots
#-------------------------------------------------------------------
plt.figure(figsize=[10,5])

plt.subplot(231)
plt.plot(t,V,'k.',t,Vfit,'m',linewidth=1.5)  
plt.xlabel('t [$\mu s$]')
plt.ylabel('V(t)')
plt.legend(['V$_{exp}$','V$_{fit}$'])

# Plot histogram of residuals
plt.subplot(234)
plt.hist(residuals,edgecolor='k',color='w',density=True)
plt.plot(resVals,stdGauss,'r',linewidth=2)
plt.legend(['Std. Gaussian','Histogram'])
plt.ylim([0,max(stdGauss)])

plt.subplot(232)
plt.plot(r,Ptruth,'k',r,Pfit,'r')
plt.fill_between(r,Pci95[:,0],Pci95[:,1],color='r',alpha=0.2,linestyle='None')
plt.fill_between(r,Pci50[:,0],Pci50[:,1],color='r',alpha=0.4,linestyle='None')
plt.xlabel('r [nm]')
plt.ylabel('P(r) [nm$^{-1}$]')
plt.legend(['P_{truth}','P_{fit}','95%-CI','50%-CI'])
plt.ylim([0, 1.5])
plt.tight_layout()

plt.subplot(235)
plt.plot(r,Ptruth,'k',r,Pfit,'b')
plt.fill_between(r,Pbootci95[:,0],Pbootci95[:,1],color='b',alpha=0.2,linestyle='None')
plt.fill_between(r,Pbootci50[:,0],Pbootci50[:,1],color='b',alpha=0.4,linestyle='None')
plt.xlabel('r [nm]')
plt.ylabel('P(r) [nm$^{-1}$]')
plt.legend(['P_{truth}','P_{fit}','95%-CI','50%-CI'])
plt.ylim([0, 1.5])
plt.tight_layout()

plt.subplot(233)
plt.plot(parVals1,P1,'r',linewidth=1.5)
plt.plot(values1, pdf1,'b',linewidth=1.5)   
plt.xlabel('\lambda')
plt.ylabel('Probability')
plt.legend(['Standard Distribution','Boot. Distribution'])
plt.xlim([0.3, 0.4])

plt.subplot(236)
plt.plot(parVals2,P2,'r',linewidth=1.5)
plt.plot(values2, pdf2,'b',linewidth=1.5)   
plt.xlabel('Spin concentration [uM]')
plt.ylabel('Probability')
plt.legend(['Standard Distribution','Boot. Distribution'])


# %%
